
----------------------------------------------------------------
--   qlprim.vhd 
--   VITAL Support : Version 2.2b, Level 1 Models.

--   NOTES        : TimingChecksOn(TRUE), XGenerationOn(FALSE),
--                  TimingMessage(TRUE), OnEvent
--
--   Possible Changes : 1.In P_FF - use thold_C_D_posedge for Model T instead of
--                        thold_D_C_posedge.
--                      2.Change delays back to unit delay (?)   
--
--   History      : 02/22/96  Changed the models for P_MUX2 and P_MUX3 to
--                            Truth tables.
--
----------------------------------------------------------------

----- CELL P_AND6 -----
library IEEE;
use IEEE.STD_LOGIC_1164.all;
library VITAL;
use VITAL.VITAL_Timing.all;


-- entity declaration --
entity P_AND6 is
   generic(
      TimingChecksOn: Boolean := True;
      XGenerationOn: Boolean := False;
      InstancePath: STRING := "*";
      tpd_A_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_B_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_C_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_D_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_E_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_F_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_A                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_B                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_C                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_D                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_E                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_F                         :  DelayType01 := (0.000 ns, 0.000 ns));

   port(
      A                              :  in    STD_LOGIC;
      B                              :  in    STD_LOGIC;
      C                              :  in    STD_LOGIC;
      D                              :  in    STD_LOGIC;
      E                              :  in    STD_LOGIC;
      F                              :  in    STD_LOGIC;
      Z                              :  out   STD_LOGIC);
end P_AND6;

-- architecture body --
library VITAL;
use VITAL.VITAL_Primitives.all;
architecture VITAL_ARCH of P_AND6 is
   attribute VITAL_LEVEL1 of VITAL_ARCH : architecture is TRUE;

   SIGNAL A_ipd  : STD_LOGIC := 'U';
   SIGNAL B_ipd  : STD_LOGIC := 'U';
   SIGNAL C_ipd  : STD_LOGIC := 'U';
   SIGNAL D_ipd  : STD_LOGIC := 'U';
   SIGNAL E_ipd  : STD_LOGIC := 'U';
   SIGNAL F_ipd  : STD_LOGIC := 'U';

begin

   ---------------------
   --  INPUT PATH DELAYs
   ---------------------
   wire_delay : block
   begin
   VitalPropagateWireDelay (A_ipd, A, VitalExtendToFillDelay(tipd_A));
   VitalPropagateWireDelay (B_ipd, B, VitalExtendToFillDelay(tipd_B));
   VitalPropagateWireDelay (C_ipd, C, VitalExtendToFillDelay(tipd_C));
   VitalPropagateWireDelay (D_ipd, D, VitalExtendToFillDelay(tipd_D));
   VitalPropagateWireDelay (E_ipd, E, VitalExtendToFillDelay(tipd_E));
   VitalPropagateWireDelay (F_ipd, F, VitalExtendToFillDelay(tipd_F));
   end block;
   --------------------
   --  BEHAVIOR SECTION
   --------------------
   VITALBehavior : process (A_ipd, B_ipd, C_ipd, D_ipd, E_ipd, F_ipd)


   -- functionality results
   VARIABLE Results : STD_LOGIC_VECTOR(1 to 1) := (others => 'X');
   ALIAS Z_zd : STD_LOGIC is Results(1);

   -- output glitch detection variables
   VARIABLE Z_GlitchData        : GlitchDataType;

   begin

      -------------------------
      --  Functionality Section
      -------------------------
      Z_zd :=
       ((NOT B_ipd)) AND (A_ipd) AND (C_ipd) AND ((NOT D_ipd)) AND (E_ipd)
         AND ((NOT F_ipd));

      ----------------------
      --  Path Delay Section
      ----------------------
      VitalPropagatePathDelay (
       OutSignal => Z,
       OutSignalName => "Z",
       OutTemp => Z_zd,
       Paths => (0 => (A_ipd'last_event, VitalExtendToFillDelay(tpd_A_Z), TRUE),
                 1 => (B_ipd'last_event, VitalExtendToFillDelay(tpd_B_Z), TRUE),
                 2 => (C_ipd'last_event, VitalExtendToFillDelay(tpd_C_Z), TRUE),
                 3 => (D_ipd'last_event, VitalExtendToFillDelay(tpd_D_Z), TRUE),
                 4 => (E_ipd'last_event, VitalExtendToFillDelay(tpd_E_Z), TRUE),
                 5 => (F_ipd'last_event, VitalExtendToFillDelay(tpd_F_Z),
TRUE)),
       GlitchData => Z_GlitchData,
       GlitchMode => NoGlitch,
       GlitchKind => OnEvent);


end process;

end VITAL_ARCH;

configuration CFG_P_AND6_VITAL of P_AND6 is
   for VITAL_ARCH
   end for;
end CFG_P_AND6_VITAL;


----- CELL P_BUF -----
library IEEE;
use IEEE.STD_LOGIC_1164.all;
library VITAL;
use VITAL.VITAL_Timing.all;


-- entity declaration --
entity P_BUF is
   generic(
      TimingChecksOn: Boolean := True;
      XGenerationOn: Boolean := False;
      InstancePath: STRING := "*";
      tpd_A_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_A                         :  DelayType01 := (0.000 ns, 0.000 ns));

   port(
      A                              :  in    STD_LOGIC;
      Z                              :  out   STD_LOGIC);
end P_BUF;

-- architecture body --
library VITAL;
use VITAL.VITAL_Primitives.all;
architecture VITAL_ARCH of P_BUF is
   attribute VITAL_LEVEL1 of VITAL_ARCH : architecture is TRUE;

   SIGNAL A_ipd  : STD_LOGIC := 'U';

begin

   ---------------------
   --  INPUT PATH DELAYs
   ---------------------
   wire_delay : block
   begin
   VitalPropagateWireDelay (A_ipd, A, VitalExtendToFillDelay(tipd_A));
   end block;
   --------------------
   --  BEHAVIOR SECTION
   --------------------
   VITALBehavior : process (A_ipd)


   -- functionality results
   VARIABLE Results : STD_LOGIC_VECTOR(1 to 1) := (others => 'X');
   ALIAS Z_zd : STD_LOGIC is Results(1);

   -- output glitch detection variables
   VARIABLE Z_GlitchData        : GlitchDataType;

   begin

      -------------------------
      --  Functionality Section
      -------------------------
      Z_zd := TO_X01(A_ipd);

      ----------------------
      --  Path Delay Section
      ----------------------
      VitalPropagatePathDelay (
       OutSignal => Z,
       OutSignalName => "Z",
       OutTemp => Z_zd,
       Paths => (0 => (A_ipd'last_event, VitalExtendToFillDelay(tpd_A_Z),
TRUE)),
       GlitchData => Z_GlitchData,
       GlitchMode => NoGlitch,
       GlitchKind => OnEvent);


end process;

end VITAL_ARCH;

configuration CFG_P_BUF_VITAL of P_BUF is
   for VITAL_ARCH
   end for;
end CFG_P_BUF_VITAL;


----- CELL P_FF -----
library IEEE;
use IEEE.STD_LOGIC_1164.all;
library VITAL;
use VITAL.VITAL_Timing.all;


-- entity declaration --
entity P_FF is
   generic(
      TimingChecksOn: Boolean := True;
      XGenerationOn: Boolean := False;
      InstancePath: STRING := "*";
      tpd_S_Q                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_R_Q                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_C_Q                        :  DelayType01 := (0.000 ns, 0.000 ns);
      trecovery_R_C_negedge_posedge  :  DelayTypeXX := 0.000 ns;
      trecovery_S_C_negedge_posedge  :  DelayTypeXX := 0.000 ns;
      tsetup_D_C_posedge        :  DelayTypeXX := 0.000 ns;
      thold_C_D_posedge              :  DelayTypeXX := 0.000 ns;
--      thold_D_C_posedge              :  DelayTypeXX := 0.000 ns;
--    Due to different specification in VITAL 2.2b
      tpw_C_negedge                  :  DelayTypeXX := 0.000 ns;
      tpw_C_posedge                  :  DelayTypeXX := 0.000 ns;
      tipd_D                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_C                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_S                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_R                         :  DelayType01 := (0.000 ns, 0.000 ns));

   port(
      D                              :  in    STD_LOGIC;
      C                              :  in    STD_LOGIC;
      S                              :  in    STD_LOGIC;
      R                              :  in    STD_LOGIC;
      Q                              :  out   STD_LOGIC);
end P_FF;

-- architecture body --
library VITAL;
use VITAL.VITAL_Primitives.all;
architecture VITAL_ARCH of P_FF is
   attribute VITAL_LEVEL1 of VITAL_ARCH : architecture is TRUE;

   SIGNAL D_ipd  : STD_LOGIC := 'U';
   SIGNAL C_ipd  : STD_LOGIC := 'U';
   SIGNAL S_ipd  : STD_LOGIC := 'U';
   SIGNAL R_ipd  : STD_LOGIC := 'U';

begin

   ---------------------
   --  INPUT PATH DELAYs
   ---------------------
   wire_delay : block
   begin
   VitalPropagateWireDelay (D_ipd, D, VitalExtendToFillDelay(tipd_D));
   VitalPropagateWireDelay (C_ipd, C, VitalExtendToFillDelay(tipd_C));
   VitalPropagateWireDelay (S_ipd, S, VitalExtendToFillDelay(tipd_S));
   VitalPropagateWireDelay (R_ipd, R, VitalExtendToFillDelay(tipd_R));
   end block;
   --------------------
   --  BEHAVIOR SECTION
   --------------------
   VITALBehavior : process (D_ipd, C_ipd, S_ipd, R_ipd)

   -- timing check results
   VARIABLE Tviol_D_C, Tviol_R_C, Tviol_S_C   : X01 := '0';
   VARIABLE Pviol_C : X01 := '0';
   VARIABLE Tmkr_D_C,  Tmkr_R_C,  Tmkr_S_C    : TimeMarkerType;
 --  VARIABLE PeriodCheckInfo_C : DelayArrayTypeXX(0 to 1) :=
 --                                       PeriodCheckInfo_Init;
   VARIABLE PeriodCheckInfo_C : DelayArrayTypeXX(0 to 1);

   -- functionality results
   VARIABLE Violation : X01 := '0';
   VARIABLE PrevData_Q : STD_LOGIC_VECTOR(1 to 5) := (others => 'X');
--   VARIABLE D_delayed : STD_LOGIC := 'X';
--   VARIABLE C_delayed : STD_LOGIC := 'X';
   VARIABLE Results : STD_LOGIC_VECTOR(1 to 1) := (others => 'X');
   ALIAS Q_zd : STD_LOGIC is Results(1);

   -- output glitch detection variables
   VARIABLE Q_GlitchData        : GlitchDataType;

---
   CONSTANT  L  : VitalTableSymbolType := '0';
   CONSTANT  H  : VitalTableSymbolType := '1';
   CONSTANT  X  : VitalTableSymbolType := 'X'; --Unknown
   CONSTANT  NC : VitalTableSymbolType := 'S'; -- no change
   CONSTANT  LH : VitalTableSymbolType := '/'; -- rising edge
   CONSTANT  HL : VitalTableSymbolType := '\'; -- falling edge
   CONSTANT  xH : VitalTableSymbolType := '^'; -- x to 1
   CONSTANT  Lx : VitalTableSymbolType := 'r'; -- 0 to x
   CONSTANT  xL : VitalTableSymbolType := 'v'; -- x to 0
   CONSTANT  Hx : VitalTableSymbolType := 'f'; -- 1 to x
   CONSTANT  D  : VitalTableSymbolType := '-'; -- Don't care
   CONSTANT  D0 : VitalTableSymbolType := 'N'; -- any edge to 0
   CONSTANT  V  : VitalTableSymbolType := 'X'; -- Violation 
   CONSTANT  E  : VitalTableSymbolType := '*'; -- Any edge 
   CONSTANT  PV : VitalTableSymbolType := 'S'; -- previous value

   CONSTANT P_FF_tab : VitalStateTableType (1 to 18, 1 to 7) := (
   -------------------------------------------
--Viol  D    C      R    S      Q   :   Q+
   -------------------------------------------
   (V,  D,   D,     D,   D,     D,      PV ),

   (D,  H,   LH,    L,   L,     D,      H ),
   (D,  H,   LH,    L,   X,     D,      H ),
   (D,  D,    D,    L,   X,     H,      H ),

   (D,  L,   LH,    L,   L,     D,      L ),
   (D,  L,   LH,    X,   L,     D,      L ),
   (D,  D,    D,    X,   L,     L,      L ),

   (D,  H,   xH,    L,   L,     H,      H ),
   (D,  L,   xH,    L,   L,     L,      L ),
   (D,  H,   Lx,    L,   L,     H,      H ),
   (D,  L,   Lx,    L,   L,     L,      L ),

   (D,  D,   D,     H,   D,     D,      L ),
   (D,  D,   D,     L,   H,     D,      H ),

   (D,  D,   D0,    L,   L,     D,      NC ),
   (D,  D,   Hx,    L,   L,     D,      NC ),
   (D,  E,   D,     D,   D,     D,      NC ),

   (D,  D,   D,     D0,  D,     D,      NC ),
   (D,  D,   D,     D,   D0,    D,      NC ));

   begin

      ------------------------
      --  Timing Check Section
      ------------------------
      if (TimingChecksOn) then

         VitalTimingCheck (D_ipd, "D", C_ipd, "C",
                           tsetup_D_C_posedge, tsetup_D_C_posedge,
                           --thold_D_C_posedge, thold_D_C_posedge,
                           thold_C_D_posedge, thold_C_D_posedge,
                           TO_X01(( S_ipd ) OR ( R_ipd )) /= '1',
                           TO_X01(C_ipd) = '1',
                           InstancePath & "/P_FF",
                           Tmkr_D_C, Tviol_D_C);
         VitalTimingCheck (R_ipd, "R", C_ipd, "C",
                           0 ns, trecovery_R_C_negedge_posedge, 
                           0 ns, 0 ns,
                           TO_X01( S_ipd ) /= '1',
                           TO_X01(C_ipd) = '1',
                           InstancePath & "/P_FF",
                           Tmkr_R_C, Tviol_R_C);
         VitalTimingCheck (S_ipd, "S", C_ipd, "C",
                           0 ns, trecovery_S_C_negedge_posedge, 
                           0 ns, 0 ns,
                           TO_X01( R_ipd ) /= '1',
                           TO_X01(C_ipd) = '1',
                           InstancePath & "/P_FF",
                           Tmkr_S_C, Tviol_S_C);
         VitalPeriodCheck (C_ipd, "C", 
                           4 ns, TIME'HIGH,
                           tpw_C_posedge, TIME'HIGH,
                           tpw_C_negedge, TIME'HIGH,
                           PeriodCheckInfo_C, Pviol_C,
                           InstancePath & "/P_FF", TRUE);
            
            
      end if;

      -------------------------
      --  Functionality Section
      -------------------------
      Violation := Tviol_D_C OR Tviol_R_C or Tviol_S_C or Pviol_C;

      VitalStateTable(
        StateTable => P_FF_tab,
        DataIn => (Violation, D_ipd, C_ipd, R_ipd, S_ipd),
        Result => Q_zd,
        PreviousDataIn => PrevData_Q);

      ----------------------
      --  Path Delay Section
      ----------------------
      VitalPropagatePathDelay (
       OutSignal => Q,
       OutSignalName => "Q",
       OutTemp => Q_zd,
       Paths => (0 => (S_ipd'last_event, VitalExtendToFillDelay(tpd_S_Q), TRUE),
               1 => (R_ipd'last_event, VitalExtendToFillDelay(tpd_R_Q), TRUE),
               2 => (C_ipd'last_event, VitalExtendToFillDelay(tpd_C_Q), TRUE),
               3 => (D_ipd'last_event, VitalExtendToFillDelay(tpd_C_Q), TRUE)),
       GlitchData => Q_GlitchData,
       GlitchMode => NoGlitch,
       GlitchKind => OnEvent);

end process;

end VITAL_ARCH;

configuration CFG_P_FF_VITAL of P_FF is
   for VITAL_ARCH
   end for;
end CFG_P_FF_VITAL;


----- CELL P_INV -----
library IEEE;
use IEEE.STD_LOGIC_1164.all;
library VITAL;
use VITAL.VITAL_Timing.all;


-- entity declaration --
entity P_INV is
   generic(
      TimingChecksOn: Boolean := True;
      XGenerationOn: Boolean := False;
      InstancePath: STRING := "*";
      tpd_A_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_A                         :  DelayType01 := (0.000 ns, 0.000 ns));

   port(
      A                              :  in    STD_LOGIC;
      Z                              :  out   STD_LOGIC);
end P_INV;

-- architecture body --
library VITAL;
use VITAL.VITAL_Primitives.all;
architecture VITAL_ARCH of P_INV is
   attribute VITAL_LEVEL1 of VITAL_ARCH : architecture is TRUE;

   SIGNAL A_ipd  : STD_LOGIC := 'U';

begin

   ---------------------
   --  INPUT PATH DELAYs
   ---------------------
   wire_delay : block
   begin
   VitalPropagateWireDelay (A_ipd, A, VitalExtendToFillDelay(tipd_A));
   end block;
   --------------------
   --  BEHAVIOR SECTION
   --------------------
   VITALBehavior : process (A_ipd)


   -- functionality results
   VARIABLE Results : STD_LOGIC_VECTOR(1 to 1) := (others => 'X');
   ALIAS Z_zd : STD_LOGIC is Results(1);

   -- output glitch detection variables
   VARIABLE Z_GlitchData        : GlitchDataType;

   begin

      -------------------------
      --  Functionality Section
      -------------------------
      Z_zd := (NOT A_ipd);

      ----------------------
      --  Path Delay Section
      ----------------------
      VitalPropagatePathDelay (
       OutSignal => Z,
       OutSignalName => "Z",
       OutTemp => Z_zd,
       Paths => (0 => (A_ipd'last_event, VitalExtendToFillDelay(tpd_A_Z),
TRUE)),
       GlitchData => Z_GlitchData,
       GlitchMode => NoGlitch,
       GlitchKind => OnEvent);


end process;

end VITAL_ARCH;

configuration CFG_P_INV_VITAL of P_INV is
   for VITAL_ARCH
   end for;
end CFG_P_INV_VITAL;


----- CELL P_MUX2 -----
library IEEE;
use IEEE.STD_LOGIC_1164.all;
library VITAL;
use VITAL.VITAL_Timing.all;


-- entity declaration --
entity P_MUX2 is
   generic(
      TimingChecksOn: Boolean := True;
      XGenerationOn: Boolean := False;
      InstancePath: STRING := "*";
      tpd_A_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_B_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_C_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_D_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_S_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_A                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_B                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_C                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_D                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_S                         :  DelayType01 := (0.000 ns, 0.000 ns));

   port(
      A                              :  in    STD_LOGIC;
      B                              :  in    STD_LOGIC;
      C                              :  in    STD_LOGIC;
      D                              :  in    STD_LOGIC;
      S                              :  in    STD_LOGIC;
      Z                              :  out   STD_LOGIC);
end P_MUX2;

-- architecture body --
library VITAL;
use VITAL.VITAL_Primitives.all;
architecture VITAL_ARCH of P_MUX2 is
   attribute VITAL_LEVEL1 of VITAL_ARCH : architecture is TRUE;

   SIGNAL A_ipd  : STD_LOGIC := 'U';
   SIGNAL B_ipd  : STD_LOGIC := 'U';
   SIGNAL C_ipd  : STD_LOGIC := 'U';
   SIGNAL D_ipd  : STD_LOGIC := 'U';
   SIGNAL S_ipd  : STD_LOGIC := 'U';

begin

   ---------------------
   --  INPUT PATH DELAYs
   ---------------------
   wire_delay : block
   begin
   VitalPropagateWireDelay (A_ipd, A, VitalExtendToFillDelay(tipd_A));
   VitalPropagateWireDelay (B_ipd, B, VitalExtendToFillDelay(tipd_B));
   VitalPropagateWireDelay (C_ipd, C, VitalExtendToFillDelay(tipd_C));
   VitalPropagateWireDelay (D_ipd, D, VitalExtendToFillDelay(tipd_D));
   VitalPropagateWireDelay (S_ipd, S, VitalExtendToFillDelay(tipd_S));
   end block;
   --------------------
   --  BEHAVIOR SECTION
   --------------------
   VITALBehavior : process (A_ipd, B_ipd, C_ipd, D_ipd, S_ipd)

   CONSTANT  L  : VitalTableSymbolType := '0';
   CONSTANT  H  : VitalTableSymbolType := '1';
   CONSTANT  D  : VitalTableSymbolType := '-'; -- Don't care

   CONSTANT P_MUX2_tab : VitalTruthTableType (1 to 11, 1 to 6) := (
   -------------------------------------------
   --     A   B   C   D   S        Z
   -------------------------------------------
         (H,  L,  D,  D,  L,       H  ),
         (L,  D,  D,  D,  L,       L  ),
         (D,  H,  D,  D,  L,       L  ),
         (D,  D,  H,  L,  H,       H  ),
         (D,  D,  L,  D,  H,       L  ),
         (D,  D,  D,  H,  H,       L  ),
-- Reduce pessimism
         (H,  L,  H,  L,  D,       H  ),
         (L,  D,  L,  D,  D,       L  ),
         (L,  D,  D,  H,  D,       L  ),
         (D,  H,  D,  H,  D,       L  ),
         (D,  H,  L,  D,  D,       L  ));


   -- functionality results
   VARIABLE Results : STD_LOGIC_VECTOR(1 to 1) := (others => 'X');
   ALIAS Z_zd : STD_LOGIC is Results(1);

   -- output glitch detection variables
   VARIABLE Z_GlitchData        : GlitchDataType;

   begin

      -------------------------
      --  Functionality Section
      -------------------------

   Z_zd :=  VitalTruthTable(
        TruthTable => P_MUX2_tab,
        DataIn => std_logic_vector'(A_ipd, B_ipd, C_ipd, D_ipd, S_ipd));

      ----------------------
      --  Path Delay Section
      ----------------------
      VitalPropagatePathDelay (
       OutSignal => Z,
       OutSignalName => "Z",
       OutTemp => Z_zd,
       Paths => (0 => (A_ipd'last_event, VitalExtendToFillDelay(tpd_A_Z), TRUE),
                 1 => (B_ipd'last_event, VitalExtendToFillDelay(tpd_B_Z), TRUE),
                 2 => (C_ipd'last_event, VitalExtendToFillDelay(tpd_C_Z), TRUE),
                 3 => (D_ipd'last_event, VitalExtendToFillDelay(tpd_D_Z), TRUE),
                 4 => (S_ipd'last_event, VitalExtendToFillDelay(tpd_S_Z), TRUE)
                 ),
       GlitchData => Z_GlitchData,
       GlitchMode => NoGlitch,
       GlitchKind => OnEvent);


end process;

end VITAL_ARCH;

configuration CFG_P_MUX2_VITAL of P_MUX2 is
   for VITAL_ARCH
   end for;
end CFG_P_MUX2_VITAL;


----- CELL P_MUX3 -----
library IEEE;
use IEEE.STD_LOGIC_1164.all;
library VITAL;
use VITAL.VITAL_Timing.all;


-- entity declaration --
entity P_MUX3 is
   generic(
      TimingChecksOn: Boolean := True;
      XGenerationOn: Boolean := False;
      InstancePath: STRING := "*";
      tpd_A_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_B_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_C_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_D_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_E_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_S_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_T_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_A                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_B                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_C                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_D                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_E                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_S                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_T                         :  DelayType01 := (0.000 ns, 0.000 ns));

   port(
      A                              :  in    STD_LOGIC;
      B                              :  in    STD_LOGIC;
      C                              :  in    STD_LOGIC;
      D                              :  in    STD_LOGIC;
      E                              :  in    STD_LOGIC;
      S                              :  in    STD_LOGIC;
      T                              :  in    STD_LOGIC;
      Z                              :  out   STD_LOGIC);
end P_MUX3;

-- architecture body --
library VITAL;
use VITAL.VITAL_Primitives.all;
architecture VITAL_ARCH of P_MUX3 is
   attribute VITAL_LEVEL1 of VITAL_ARCH : architecture is TRUE;

   SIGNAL A_ipd  : STD_LOGIC := 'U';
   SIGNAL B_ipd  : STD_LOGIC := 'U';
   SIGNAL C_ipd  : STD_LOGIC := 'U';
   SIGNAL D_ipd  : STD_LOGIC := 'U';
   SIGNAL E_ipd  : STD_LOGIC := 'U';
   SIGNAL S_ipd  : STD_LOGIC := 'U';
   SIGNAL T_ipd  : STD_LOGIC := 'U';

begin

   ---------------------
   --  INPUT PATH DELAYs
   ---------------------
   wire_delay : block
   begin
   VitalPropagateWireDelay (A_ipd, A, VitalExtendToFillDelay(tipd_A));
   VitalPropagateWireDelay (B_ipd, B, VitalExtendToFillDelay(tipd_B));
   VitalPropagateWireDelay (C_ipd, C, VitalExtendToFillDelay(tipd_C));
   VitalPropagateWireDelay (D_ipd, D, VitalExtendToFillDelay(tipd_D));
   VitalPropagateWireDelay (E_ipd, E, VitalExtendToFillDelay(tipd_E));
   VitalPropagateWireDelay (S_ipd, S, VitalExtendToFillDelay(tipd_S));
   VitalPropagateWireDelay (T_ipd, T, VitalExtendToFillDelay(tipd_T));
   end block;
   --------------------
   --  BEHAVIOR SECTION
   --------------------
   VITALBehavior : process (A_ipd, B_ipd, C_ipd, D_ipd, E_ipd, S_ipd, T_ipd)


   CONSTANT  L  : VitalTableSymbolType := '0';
   CONSTANT  H  : VitalTableSymbolType := '1';
   CONSTANT  D  : VitalTableSymbolType := '-'; -- Don't care

   CONSTANT P_MUX3_tab : VitalTruthTableType (1 to 24, 1 to 8) := (
   -------------------------------------------
   --   A   B   C   D   E      S   T        Z
   -------------------------------------------
      ( H,  L,  D,  D,  D,     L,  L,       H  ),
      ( L,  D,  D,  D,  D,     L,  L,       L  ),
      ( D,  H,  D,  D,  D,     L,  L,       L  ),
      ( D,  D,  H,  L,  D,     H,  L,       H  ),
      ( D,  D,  L,  D,  D,     H,  L,       L  ),
      ( D,  D,  D,  H,  D,     H,  L,       L  ),
      ( D,  D,  D,  D,  L,     D,  H,       L  ),
      ( D,  D,  D,  D,  H,     D,  H,       H  ),
-- Reduce pessimism
      ( H,  L,  H,  L,  H,     D,  D,       H  ),
      ( L,  D,  L,  D,  L,     D,  D,       L  ),
      ( L,  D,  D,  H,  L,     D,  D,       L  ),
      ( D,  H,  D,  H,  L,     D,  D,       L  ),
      ( D,  H,  L,  D,  L,     D,  D,       L  ), 
      ( H,  L,  H,  L,  D,     D,  L,       H  ),
      ( L,  D,  L,  D,  D,     D,  L,       L  ),
      ( D,  H,  D,  H,  D,     D,  L,       L  ),
      ( D,  H,  L,  D,  D,     D,  L,       L  ), 
      ( L,  D,  D,  H,  D,     D,  L,       L  ), 
      ( L,  D,  D,  D,  L,     L,  D,       L  ),
      ( D,  H,  D,  D,  L,     L,  D,       L  ),
      ( H,  L,  D,  D,  H,     L,  D,       H  ),
      ( D,  D,  L,  D,  L,     H,  D,       L  ),
      ( D,  D,  D,  H,  L,     H,  D,       L  ),
      ( D,  D,  H,  L,  H,     H,  D,       H  ));


   -- functionality results
   VARIABLE Results : STD_LOGIC_VECTOR(1 to 1) := (others => 'X');
   ALIAS Z_zd : STD_LOGIC is Results(1);

   -- output glitch detection variables
   VARIABLE Z_GlitchData        : GlitchDataType;

   begin

      -------------------------
      --  Functionality Section
      -------------------------

      Z_zd :=  VitalTruthTable(
          TruthTable => P_MUX3_tab,
               DataIn => std_logic_vector'(A_ipd, B_ipd, C_ipd, D_ipd, 
                                           E_ipd, S_ipd, T_ipd));
      ----------------------
      --  Path Delay Section
      ----------------------
      VitalPropagatePathDelay (
       OutSignal => Z,
       OutSignalName => "Z",
       OutTemp => Z_zd,
       Paths => (0 => (A_ipd'last_event, VitalExtendToFillDelay(tpd_A_Z), TRUE),
                 1 => (B_ipd'last_event, VitalExtendToFillDelay(tpd_B_Z), TRUE),
                 2 => (C_ipd'last_event, VitalExtendToFillDelay(tpd_C_Z), TRUE),
                 3 => (D_ipd'last_event, VitalExtendToFillDelay(tpd_D_Z), TRUE),
                 4 => (E_ipd'last_event, VitalExtendToFillDelay(tpd_E_Z), TRUE),
                 5 => (S_ipd'last_event, VitalExtendToFillDelay(tpd_S_Z), TRUE),
                 6 => (T_ipd'last_event, VitalExtendToFillDelay(tpd_T_Z),
TRUE)),
       GlitchData => Z_GlitchData,
       GlitchMode => NoGlitch,
       GlitchKind => OnEvent);


end process;

end VITAL_ARCH;

configuration CFG_P_MUX3_VITAL of P_MUX3 is
   for VITAL_ARCH
   end for;
end CFG_P_MUX3_VITAL;


----- CELL P_OUT -----
library IEEE;
use IEEE.STD_LOGIC_1164.all;
library VITAL;
use VITAL.VITAL_Timing.all;


-- entity declaration --
entity P_OUT is
   generic(
      TimingChecksOn: Boolean := True;
      XGenerationOn: Boolean := False;
      InstancePath: STRING := "*";
      tpd_A_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_B_Z                        :  DelayType01 := (0.000 ns, 0.000 ns);
      tpd_E_Z                        :  DelayType01z :=
               (0.000 ns, 0.000 ns, 0.000 ns, 0.000 ns, 0.000 ns, 0.000 ns);
      tipd_A                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_B                         :  DelayType01 := (0.000 ns, 0.000 ns);
      tipd_E                         :  DelayType01 := (0.000 ns, 0.000 ns));

   port(
      A                              :  in    STD_LOGIC;
      B                              :  in    STD_LOGIC;
      E                              :  in    STD_LOGIC;
      Z                              :  out   STD_LOGIC);
end P_OUT;

-- architecture body --
library VITAL;
use VITAL.VITAL_Primitives.all;
architecture VITAL_ARCH of P_OUT is
   attribute VITAL_LEVEL1 of VITAL_ARCH : architecture is TRUE;

   SIGNAL A_ipd  : STD_LOGIC := 'U';
   SIGNAL B_ipd  : STD_LOGIC := 'U';
   SIGNAL E_ipd  : STD_LOGIC := 'U';

begin

   ---------------------
   --  INPUT PATH DELAYs
   ---------------------
   wire_delay : block
   begin
   VitalPropagateWireDelay (A_ipd, A, VitalExtendToFillDelay(tipd_A));
   VitalPropagateWireDelay (B_ipd, B, VitalExtendToFillDelay(tipd_B));
   VitalPropagateWireDelay (E_ipd, E, VitalExtendToFillDelay(tipd_E));
   end block;
   --------------------
   --  BEHAVIOR SECTION
   --------------------
   VITALBehavior : process (A_ipd, B_ipd, E_ipd)


   -- functionality results
   VARIABLE Results : STD_LOGIC_VECTOR(1 to 1) := (others => 'X');
   ALIAS Z_zd : STD_LOGIC is Results(1);

   -- output glitch detection variables
   VARIABLE Z_GlitchData        : GlitchDataType;

   begin

      -------------------------
      --  Functionality Section
      -------------------------
      Z_zd := VitalBUFIF0 (data => (B_ipd) OR ((NOT A_ipd)),
              enable => (NOT E_ipd));

      ----------------------
      --  Path Delay Section
      ----------------------
      VitalPropagatePathDelay (
       OutSignal => Z,
       OutSignalName => "Z",
       OutTemp => Z_zd,
       Paths => (0 => (A_ipd'last_event, VitalExtendToFillDelay(tpd_A_Z), TRUE),
                 1 => (B_ipd'last_event, VitalExtendToFillDelay(tpd_B_Z), TRUE),
                 2 => (E_ipd'last_event, VitalExtendToFillDelay(tpd_E_Z),
TRUE)),
       GlitchData => Z_GlitchData,
       GlitchMode => NoGlitch,
       GlitchKind => OnEvent);


end process;

end VITAL_ARCH;

configuration CFG_P_OUT_VITAL of P_OUT is
   for VITAL_ARCH
   end for;
end CFG_P_OUT_VITAL;


